(function () {
  Drupal.behaviors.maplibre_gl = {
    attach: function (context, settings) {
      const newSourceEvenet = new Event('new-event');
      const maplibreGlMapEvenet = new Event('maplibre-gl.map');

      Drupal.maplibregl_popupContents = (feature,point_coords) => {
        var coordinates = feature.geometry.coordinates.slice();
        // When using marker we cannot use lngLat as in 
        // map click event. So treat it differently.
        if (!point_coords) {
          point_coords = {
            lat: coordinates[1],
            lng: coordinates[0]
          }
        }
        const geoUrl = `geo:${point_coords.lat},${point_coords.lng}`
        var description = feature.properties.description +
          '<span><a href="' + geoUrl + '" target="_blank">'+ Drupal.t("Open in app") + '</a></span>'
        // Ensure that if the map is zoomed out such that multiple
        // copies of the feature are visible, the popup appears
        // over the copy being pointed to.
        while (Math.abs(point_coords.lng - coordinates[0]) > 180) {
          coordinates[0] += point_coords.lng > coordinates[0] ? 360 : -360;
        }
        return {
          "coordinates" : coordinates,
          "html" : description,
        };
      }

      // Loop through each on the page.
      for (const [mapId, mapData] of Object.entries(settings.maplibreGl)) {
        if (context.querySelector('#' + mapData.options.container) && !context.querySelector('#' + mapData.options.container).classList.contains('maplibre-gl-processed')) {
          const mapContainer = context.querySelector('#' + mapData.options.container);
          mapContainer.parentElement.style.height = mapData.config.height;

          var popupStyle = mapData.config.popup;

          maplibregl.accessToken = mapData.accessToken;

          // Create the map.
          var map = new maplibregl.Map(mapData.options);


          const newMapEvent = new CustomEvent('new-map', {bubbles: true, detail: {map: map, mapId: mapId}});
          mapContainer.dispatchEvent(newMapEvent);

          // Wait until map has loaded.
          map.on('load', function () {

            // Loop through sources.
            for (const [sourceId, sourceData] of Object.entries(mapData.sources || {})) {
              map.addSource(sourceId, sourceData);
              const newSourceEvent = new CustomEvent('new-source', {bubbles: true, detail: {map: map, mapData: mapData, mapId: mapId, sourceId: sourceId, sourceData: sourceData}});
              mapContainer.dispatchEvent(newSourceEvent);
            }

            // Loop through the layers.
            for (const [index, layerData] of Object.entries(mapData.layers || {})) {
              map.addLayer(layerData);
              const newLayerEvent = new CustomEvent('new-layer', {bubbles: true, detail : {map: map, mapId: mapId, layerData: layerData}});

              mapContainer.dispatchEvent(newLayerEvent);

              var link = document.createElement('a');
              link.href = '#';
              link.className = 'active';
              link.classList.add('active');
              if (layerData.source.data && layerData.source.data.features) {
                link.hidden = layerData.source.data.features.length < 1;
              } else {
                map.getSource(layerData.source).getData()
                .then((data) => {
                  link.hidden = data.features.length < 1;
                })
                .catch((reason) => {
                  link.hidden = true;
                })
              }
              link.textContent = layerData.name;
              link.dataset.layerId = layerData.id;

              link.onclick = function (e) {
                var clickedLayer = this.dataset.layerId;
                e.preventDefault();
                e.stopPropagation();

                var visibility = map.getLayoutProperty(clickedLayer, 'visibility');

                if (visibility === 'visible') {
                  map.setLayoutProperty(clickedLayer, 'visibility', 'none');
                  this.className = '';
                } else {
                  this.className = 'active';
                  map.setLayoutProperty(clickedLayer, 'visibility', 'visible');
                }
              };

              var layers = context.querySelector('#' + mapData.options.container + '-menu');
              layers.appendChild(link);

              // Add popups.
              map.on('click', layerData.id, async (e) => {
                for (const feature of e.features) {
                  const is_cluster = ["has", "point_count"].length === feature.layer.filter.length &&  ["has", "point_count"].every(function(value, index) { return value === feature.layer.filter[index]})
                  if (is_cluster){
                    const cluster_features = map.queryRenderedFeatures(e.point, {
                      layers: ['clusters']
                    });
                    const clusterId = cluster_features[0].properties.cluster_id;
                    const zoom = await map.getSource('points').getClusterExpansionZoom(clusterId);
                    map.easeTo({
                      center: cluster_features[0].geometry.coordinates,
                      zoom
                    });
                    return;
                  }
                  const popupContents = Drupal.maplibregl_popupContents(feature, e.lngLat);
                  // Check for popup style.
                  if (!popupStyle) {
                    new maplibregl.Popup()
                      .setLngLat(popupContents.coordinates)
                      .setHTML(popupContents.html)
                      .addTo(map);
                  }
                  // Bind to a different container.
                  else if (popupStyle) {
                    document.getElementById(popupStyle).innerHTML = popupContents.html;
                  }
                }
              });

              // Change the cursor to a pointer when the mouse is over the layer.
              map.on('mouseenter', layerData.id, function () {
                map.getCanvas().style.cursor = 'pointer';
              });

              // Change it back to a pointer when it leaves.
              map.on('mouseleave', layerData.id, function () {
                map.getCanvas().style.cursor = '';
              });

            }

            const newMapLibreGlMapEvent = new CustomEvent('maplibre-gl.map', {bubbles: true, detail : {map: map, mapData: mapData, mapId: mapId}});
            mapContainer.dispatchEvent(newMapLibreGlMapEvent);
          }); // Map on load.

          // Add some controls.
          for (const [controlType, controlOptions] of Object.entries(mapData.config.controls || {})) {
            // NavigationControl options are added differently.
            if (controlType == 'NavigationControl') {
              map.addControl(new maplibregl[controlType](), controlOptions);
            } else if (controlType == 'MaplibreGeocoder') {
              map.addControl(new MaplibreGeocoder(controlOptions));
            } else {
              map.addControl(new maplibregl[controlType](controlOptions));
            }
          }
          mapContainer.classList.add('maplibre-gl-processed');
        }
      };

    }

  };
})();
