<?php

/**
 * @file
 * API documentation.
 */

/**
 * Define one or more map definitions to be used when rendering a map.
 *
 * The maplibre_gl_get_info() will load each map and the returned
 * array is then passed to maplibre_gl_render_map() along with sources, layers
 * and options.
 *
 * The options array maps to the options available to Maplibre GL map object,
 * see https://maplibre.org/maplibre-gl-js/docs/API/
 *
 * @return array
 *   The map definitions.
 */
function hook_maplibre_gl_map_info() {
  // For options, see: https://maplibre.org/maplibre-gl-js/docs/API/
  $map = [
    'access_token' => 'pk.zzzzIjoiZGlnaXRhbGlua3Rhcxxxxxx1234iYSI6ImNqMTVsbmFmazAxZHoycW83M3prY2Fhb3UifQ._eVTUEEMHeaQ3QY0gQYmYw',
    'options' => [
      'container' => 'maplibre-streets',
      'style' => 'maplibre://styles/maplibre/streets-v9',
      'zoom' => 6,
      'center' => [146.315918, -41.640079],
    ],
    'config' =>
    [
      'height' => '400px',
      'popup' => 'popup',
      'controls' =>
      [
        'NavigationControl' => 'top-left',
        'AttributionControl' => [
          'compact' => TRUE,
        ],
        'MaplibreGeocoder' => [
          'accessToken' => 'pk.zzzzIjoiZGlnaXRhbGlua3Rhcxxxxxx1234iYSI6ImNqMTVsbmFmazAxZHoycW83M3prY2Fhb3UifQ._eVTUEEMHeaQ3QY0gQYmYw',
        ],
      ],
    ],
    'layers' =>
    [
      [
        'id' => 'Ports',
        'type' => 'circle',
        'source' => [
          'type' => 'geojson',
          'data' => 'https://d2ad6b4ur7yvpq.cloudfront.net/naturalearth-3.3.0/ne_10m_ports.geojson',
        ],
      ],
      [
        'id' => 'Terrain',
        'type' => 'line',
        'source' =>
        [
          'type' => 'vector',
          'url' => 'maplibre://maplibre.maplibre-terrain-v2',
        ],
        'source-layer' => 'contour',
        'layout' => [
          'line-join' => 'round',
          'line-cap' => 'round',
        ],
        'paint' => [
          'line-color' => '#ff69b4',
          'line-width' => 1,
        ],
      ],
      [
        'id' => 'My Locations',
        'type' => 'circle',
        'source' => [
          'type' => 'vector',
          'url' => 'maplibre://companyname.cj9s1qkic03un32plq1klqfms-5c4qn',
        ],
        'source-layer' => 'My_Locations_Tileset',
      ],
    ],
  ];
  return [
    'mymodule' => $map,
  ];
}

/**
 * Alters the map definitions.
 *
 * For one or more maps that were defined by
 * hook_maplibre_map_info().
 *
 * See:  https://maplibre.org/maplibre-gl-js/docs/API/
 *
 * @param array $map_info
 *   The map info array to be altered.
 */
function hook_maplibre_gl_map_info_alter(array &$map_info) {
  // Change the label.
  $map_info['mymodule']['label'] = "My New Streets Page";
  // change the style
  $map_info['maplibre_canvas_block']['options']['style'] = 'https://geoserveis.icgc.cat/styles/icgc_orto_hibrida.json';
  // Add image icon based on field value (need to be set in views)
  // not declaring the icon-image in layer layout will lead to place a marker.
  /*$map_info['maplibre_canvas_block']['layers'][0]['layout'] = [
    'icon-image' => [
      'coalesce',
      [ 'image', ['get', 'field_material_icon' ]], //'{field_material_icon}'
      [ 'image', 'marker_15'],
    ],
    'icon-overlap' => 'always',
    'icon-size' => 2,
    'text-field' => '{name}',
    'text-offset' => [0, 1],
    'text-font' => ['Open Sans Semibold']//, 'Arial Unicode MS Bold'],
  ];*/

  $map_info['maplibre_canvas_block']['layers'][0]['layout'] =[
    'text-font' => ['Open Sans Semibold'],
    'text-field' => '{name}',
    'text-offset' => [0, 2.5],
    'text-size' => 13,
    'text-font' => ['Open Sans Semibold']
  ];
  // Define the text props for painted items
  $map_info['maplibre_canvas_block']['layers'][0]['paint'] = [
    'text-color' => '#202',
    'text-halo-color' => '#fff',
    'text-halo-width'=> 2,
    "icon-color" => "#2E7831",
    "icon-halo-color" => "#ffffff",
    "icon-halo-width" => 10,
  ];

  // Set the layer type
  $map_info['maplibre_canvas_block']['layers'][0]['type'] = 'symbol';
  // cluster settings, unset($map_info['maplibre_canvas_block']['layers'][3])
  // to disable
  // Change cluster text font
  $map_info['maplibre_canvas_block']['layers'][3]['layout']['text-font'] = ['Open Sans Semibold'];


}
